# Generated by Django 2.2.12 on 2020-06-05 06:44

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('groupmanagement', '0013_fix_requestlog_date_field'),
        ('fittings', '0004_add_item_category_and_group'),
    ]

    operations = [
        migrations.CreateModel(
            name='UniCategory',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255)),
                ('color', models.TextField(max_length=20)),
                ('doctrines', models.ManyToManyField(related_name='category', to='fittings.Doctrine')),
                ('fittings', models.ManyToManyField(related_name='category', to='fittings.Fitting')),
                ('groups', models.ManyToManyField(help_text='Groups listed here will be able to access fits and doctrines listed under this category. If a category has no groups listed then it is considered an public category, accessible to anyone with permission to access permissions to the fittings module.', related_name='access_restricted_category', to='groupmanagement.AuthGroup')),
            ],
            options={
                'default_permissions': (),
            },
        ),
    ]
