# Generated by Django 2.2.12 on 2020-06-05 07:35

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('fittings', '0006_auto_20200605_0724'),
    ]

    operations = [
        migrations.AlterField(
            model_name='unicategory',
            name='doctrines',
            field=models.ManyToManyField(blank=True, help_text="All fittings in a doctrine will be treated as if they are in the doctrine's category.", related_name='category', to='fittings.Doctrine'),
        ),
        migrations.AlterField(
            model_name='unicategory',
            name='fittings',
            field=models.ManyToManyField(blank=True, help_text="Fittings only need to be tagged with a category if they are not included in any doctrines, or if they need to be labled in addition to their doctrine's category.", related_name='category', to='fittings.Fitting'),
        ),
        migrations.AlterField(
            model_name='unicategory',
            name='groups',
            field=models.ManyToManyField(blank=True, help_text='Groups listed here will be able to access fits and doctrines listed under this category. If a category has no groups listed then it is considered an public category, accessible to anyone with permission to access permissions to the fittings module.', related_name='access_restricted_category', to='groupmanagement.AuthGroup'),
        ),
    ]
