# Generated by Django 2.2.12 on 2020-06-05 07:24
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('fittings', '0005_unicategory'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='unicategory',
            options={'default_permissions': (), 'verbose_name': 'Unified Category', 'verbose_name_plural': 'Unified Categories'},
        ),
    ]
